/*================================================================================
	
		************************************************
		***** [Team Semiclip: File Creator 1.0.0] ******
		************************************************
	
	----------------------
	-*- Licensing Info -*-
	----------------------
	
	Team Semiclip: File Creator
	by schmurgel1983(@msn.com)
	Copyright (C) 2014 Stefan "schmurgel1983" Focke
	
	This program is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation, either version 3 of the License, or (at your
	option) any later version.
	
	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
	Public License for more details.
	
	You should have received a copy of the GNU General Public License along
	with this program. If not, see <http://www.gnu.org/licenses/>.
	
	In addition, as a special exception, the author gives permission to
	link the code of this program with the Half-Life Game Engine ("HL
	Engine") and Modified Game Libraries ("MODs") developed by Valve,
	L.L.C ("Valve"). You must obey the GNU General Public License in all
	respects for all of the code used other than the HL Engine and MODs
	from Valve. If you modify this file, you may extend this exception
	to your version of the file, but you are not obligated to do so. If
	you do not wish to do so, delete this exception statement from your
	version.
	
	No warranties of any kind. Use at your own risk.
	
	-------------------
	-*- Description -*-
	-------------------
	
	-----
	
	--------------------
	-*- Requirements -*-
	--------------------
	
	* Mods: Counter-Strike 1.6, Condition-Zero or Day of Defeat
	* Metamod: Version 1.21 or later
	* AMXX: Version 1.8.2 or later
	* Module: engine, fakemeta
	
	----------------
	-*- Commands -*-
	----------------
	
	-----
	
	---------------------
	-*- Configuration -*-
	---------------------
	
	-----
	
	---------------
	-*- Credits -*-
	---------------
	
	-----
	
	-----------------
	-*- Changelog -*-
	-----------------
	
	+ Added feature or noticeable improvement
	- Bug fix or something removed
	* Changed or Information
	! Important - Take note!
	PCS = Plugin Customization Section
	
	* v1.0.0:
		* initial release
		* supported entities:
			+ func_button
			+ func_door
			+ func_door_rotating
			+ func_guntarget
			+ func_pendulum
			+ func_plat
			+ func_platrot
			+ func_rot_button
			+ func_rotating
			+ func_tank
			+ func_trackchange
			+ func_tracktrain
			+ func_train
			+ func_vehicle
			+ momentary_door
			+ momentary_rot_button
	
	+ Added feature or noticeable improvement
	- Bug fix or something removed
	* Changed or Information
	! Important - Take note!
	PCS = Plugin Customization Section
	
=================================================================================*/

/*================================================================================
 [Plugin Customization]
=================================================================================*/

#define MAX_STORAGE 64	/* Max stored entities per func_*  64 */

/*================================================================================
 Customization ends here! Yes, that's it. Editing anything beyond
 here is not officially supported. Proceed at your own risk...
=================================================================================*/

#include <amxmodx>

#if AMXX_VERSION_NUM < 182
	#assert AMX Mod X v1.8.2 or later library required!
#endif

#include <engine>
#include <fakemeta>

/*================================================================================
 [TODO]
 
 ! nothing
 
=================================================================================*/

/*================================================================================
 [Constants, Offsets and Defines]
=================================================================================*/

new const PLUGIN_VERSION[] = "1.0.0"

/* All supported entity classes */
new const ENTITY_CLASSES[][] =
{
	"func_button",
	"func_door",
	"func_door_rotating",
	"func_guntarget",
	"func_pendulum",
	"func_plat",
	"func_platrot",
	"func_rot_button",
	"func_rotating",
	"func_tank",
	"func_trackchange",
	"func_tracktrain",
	"func_train",
	"func_vehicle",
	"momentary_door",
	"momentary_rot_button"
}
const CLASS_SIZE = sizeof(ENTITY_CLASSES)

#define SF_VEHICLE_PASSABLE  0x0008
#define SF_ROTATING_NOTSOLID 0x0040

/*================================================================================
 [Global Variables]
=================================================================================*/

/* Server Global */
new g_szMapFile[96],
	g_szMapName[32]

new g_iEntityNum[CLASS_SIZE],
	g_iEntityIndex[CLASS_SIZE][MAX_STORAGE],
	g_iEntityEnable[CLASS_SIZE][MAX_STORAGE],
	g_iEntityHandle[CLASS_SIZE][MAX_STORAGE]

new g_szEntityModel[CLASS_SIZE][MAX_STORAGE][6]

/*================================================================================
 [Natives, Init and Cfg]
=================================================================================*/

public plugin_init()
{
	/* Check team semiclip and register plugin */
	CheckTeamSemiclip()
	
	/* AMX Mod X check */
	CheckAmxxVersion()
	
	/* Store Map name and Map file */
	get_mapname(g_szMapName, charsmax(g_szMapName))
	get_datadir(g_szMapFile, charsmax(g_szMapFile))
	format(g_szMapFile, charsmax(g_szMapFile), "%s/tsc/%s.tsc", g_szMapFile, g_szMapName)
}

public plugin_cfg()
{
	/* Get all entities */
	for (new iIndex = 0, iEntity, iNum; iIndex < CLASS_SIZE; iIndex++)
	{
		/* Pre-Define entities, accurate is not 100% */
		iEntity = -1, iNum = 0
		while ((iEntity = find_ent_by_class(iEntity, ENTITY_CLASSES[iIndex])) != 0)
		{
			/* Set Entity Index */
			g_iEntityIndex[iIndex][iNum] = iEntity
			
			/* Get BSP Model */
			pev(iEntity, pev_model, g_szEntityModel[iIndex][iNum], charsmax(g_szEntityModel[][]))
			
			/* Setup Entity Data */
			switch (ENTITY_CLASSES[iIndex][5])
			{
				case 'b': /* func_button */
				{
					if (pev(iEntity, pev_spawnflags) & SF_BUTTON_DONTMOVE)
					{
						g_iEntityEnable[iIndex][iNum] = 0
					}
					else
					{
						g_iEntityEnable[iIndex][iNum] = 1
					}
					g_iEntityHandle[iIndex][iNum] = 0
				}
				case 'd':
				{
					/* func_door */
					if (pev(iEntity, pev_spawnflags) & SF_DOOR_PASSABLE)
					{
						g_iEntityEnable[iIndex][iNum] = 0
					}
					else
					{
						g_iEntityEnable[iIndex][iNum] = 1
					}
					
					/* func_door_rotating */
					if (ENTITY_CLASSES[iIndex][10] == 'r')
					{
						g_iEntityHandle[iIndex][iNum] = 1
					}
					else
					{
						g_iEntityHandle[iIndex][iNum] = 0
					}
				}
				case 'g': /* func_guntarget */
				{
					g_iEntityEnable[iIndex][iNum] = 1
					g_iEntityHandle[iIndex][iNum] = 0
				}
				case 'p':
				{
					/* func_pendulum */
					if (ENTITY_CLASSES[iIndex][9] == 'u')
					{
						if (pev(iEntity, pev_spawnflags) & SF_DOOR_PASSABLE)
						{
							g_iEntityEnable[iIndex][iNum] = 0
						}
						else
						{
							g_iEntityEnable[iIndex][iNum] = 1
						}
					}
					else /* func_plat, func_platrot */
					{
						g_iEntityEnable[iIndex][iNum] = 1
					}
					g_iEntityHandle[iIndex][iNum] = 0
				}
				case 'r':
				{
					/* func_rot_button */
					if (ENTITY_CLASSES[iIndex][9] == 'b')
					{
						if (pev(iEntity, pev_spawnflags) & SF_ROTBUTTON_NOTSOLID)
						{
							g_iEntityEnable[iIndex][iNum] = 0
						}
						else
						{
							g_iEntityEnable[iIndex][iNum] = 1
						}
						g_iEntityHandle[iIndex][iNum] = 1
					}
					else /* func_rotating */
					{
						if (pev(iEntity, pev_spawnflags) & SF_ROTATING_NOTSOLID)
						{
							g_iEntityEnable[iIndex][iNum] = 0
						}
						else
						{
							g_iEntityEnable[iIndex][iNum] = 1
						}
						g_iEntityHandle[iIndex][iNum] = 0
					}
				}
				case 't':
				{
					switch (ENTITY_CLASSES[iIndex][10])
					{
						case 'd': /* momentary_door */
						{
							if (pev(iEntity, pev_spawnflags) & SF_DOOR_PASSABLE)
							{
								g_iEntityEnable[iIndex][iNum] = 0
							}
							else
							{
								g_iEntityEnable[iIndex][iNum] = 1
							}
							g_iEntityHandle[iIndex][iNum] = 0
						}
						case 'r': /* momentary_rot_button */
						{
							if (pev(iEntity, pev_spawnflags) & SF_MOMENTARY_DOOR)
							{
								g_iEntityEnable[iIndex][iNum] = 1
							}
							else
							{
								g_iEntityEnable[iIndex][iNum] = 0
							}
							g_iEntityHandle[iIndex][iNum] = 0
						}
						default:
						{
							if (ENTITY_CLASSES[iIndex][6] == 'a') /* func_tank */
							{
								g_iEntityEnable[iIndex][iNum] = 1
								g_iEntityHandle[iIndex][iNum] = 1
							}
							else if (ENTITY_CLASSES[iIndex][10] == 'c') /* func_trackchange */
							{
								g_iEntityEnable[iIndex][iNum] = 1
								g_iEntityHandle[iIndex][iNum] = 0
							}
							else /* func_tracktrain, func_train */
							{
								if (pev(iEntity, pev_spawnflags) & SF_TRAIN_PASSABLE)
								{
									g_iEntityEnable[iIndex][iNum] = 0
								}
								else
								{
									g_iEntityEnable[iIndex][iNum] = 1
								}
								g_iEntityHandle[iIndex][iNum] = 0
							}
						}
					}
				}
				case 'v': /* func_vehicle */
				{
					if (pev(iEntity, pev_spawnflags) & SF_VEHICLE_PASSABLE)
					{
						g_iEntityEnable[iIndex][iNum] = 0
					}
					else
					{
						g_iEntityEnable[iIndex][iNum] = 1
					}
					g_iEntityHandle[iIndex][iNum] = 0
				}
			}
			
			if (++iNum >= MAX_STORAGE)
				break
		}
		g_iEntityNum[iIndex] = iNum
	}
	
	/* Save */
	SaveMapFile()
}

/*================================================================================
 [Other Functions and Tasks]
=================================================================================*/

CheckTeamSemiclip()
{
	new szModName[8], szPluginName[28]
	get_modname(szModName, charsmax(szModName))
	if (equal(szModName, "cstrike"))
	{
		szPluginName = "[CS] TSC File Creator"
		
		Label_CStrike:
		register_plugin(szPluginName, PLUGIN_VERSION, "schmurgel1983")
		
		if (LibraryExists("cs_team_semiclip", LibType_Library) || LibraryExists("zp_team_semiclip", LibType_Library))
		{
			return
		}
		set_fail_state("Error: Team Semiclip is not running!")
	}
	else if (equal(szModName, "czero"))
	{
		szPluginName = "[CZ] TSC File Creator"
		
		goto Label_CStrike
	}
	else if (equal(szModName, "dod"))
	{
		szPluginName = "[DoD] TSC File Creator"
		register_plugin(szPluginName, PLUGIN_VERSION, "schmurgel1983")
		
		if (LibraryExists("dod_team_semiclip", LibType_Library))
			return
		
		set_fail_state("Error: Team Semiclip is not running!")
	}
	
	register_plugin("[??] TSC File Creator", PLUGIN_VERSION, "schmurgel1983")
	set_fail_state("Error: This plugin is for cstrike, czero and dod only!")
}

CheckAmxxVersion()
{
	new szBuffer[6]
	get_amxx_verstring(szBuffer, charsmax(szBuffer))
	
	while (replace(szBuffer, charsmax(szBuffer), ".", "")) {}
	
	if (str_to_num(szBuffer) < 182)
		set_fail_state("Error: AMX Mod X v1.8.2 or later required!")
}

SaveMapFile()
{
	if (file_exists(g_szMapFile))
	{
		return
	}
	
	new iFile = fopen(g_szMapFile, "wt")
	if (!iFile)
	{
		return
	}
	
	/* Info */
	new szBuffer[96]
	format(szBuffer, charsmax(szBuffer), "// Map: %s^n//^n// func *model semiclip handle^n^n", g_szMapName)
	fputs(iFile, szBuffer)
	
	for (new i = 0, j, iNum; i < CLASS_SIZE; i++)
	{
		iNum = g_iEntityNum[i]
		
		if (!iNum)
		{
			continue
		}
		
		for (j = 0; j < iNum; j++)
		{
			if (pev_valid(g_iEntityIndex[i][j]))
			{
				format(szBuffer, charsmax(szBuffer), "%s %s %s %s^n", ENTITY_CLASSES[i], g_szEntityModel[i][j], g_iEntityEnable[i][j] ? "enable" : "ignore", g_iEntityHandle[i][j] ? "stay" : "move")
				fputs(iFile, szBuffer)
			}
		}
	}
	fclose(iFile)
}

/*================================================================================
 [Stocks]
=================================================================================*/

stock get_datadir(name[],len)
{
	return get_localinfo("amxx_datadir",name,len);
}
