/*================================================================================
	
		**********************************************
		*********** [No engine knockback] ************
		**********************************************
	
	----------------------
	-*- Licensing Info -*-
	----------------------
	
	No engine knockback
	by schmurgel1983(@msn.com)
	Copyright (C) Stefan "schmurgel1983" Focke
	
	This program is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation, either version 3 of the License, or (at your
	option) any later version.
	
	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
	Public License for more details.
	
	You should have received a copy of the GNU General Public License along
	with this program. If not, see <http://www.gnu.org/licenses/>.
	
	In addition, as a special exception, the author gives permission to
	link the code of this program with the Half-Life Game Engine ("HL
	Engine") and Modified Game Libraries ("MODs") developed by Valve,
	L.L.C ("Valve"). You must obey the GNU General Public License in all
	respects for all of the code used other than the HL Engine and MODs
	from Valve. If you modify this file, you may extend this exception
	to your version of the file, but you are not obligated to do so. If
	you do not wish to do so, delete this exception statement from your
	version.
	
	No warranties of any kind. Use at your own risk.
	
	-------------------
	-*- Description -*-
	-------------------
	
	This plugin disable the cs/cz engine knockback, but not the ZP knockback.
	The cs/cz engine knockback is little but noticeable.
	
	---------------------
	-*- Configuration -*-
	---------------------
	
	zpnm_nek_nemesis 0 // Nemesis don't have engine knockback [0-disabled / 1-enabled]
	zpnm_nek_zombies 1 // Zombies don't have engine knockback [0-disabled / 1-enabled]
	zpnm_nek_assassins 1 // Assassins don't have engine knockback [0-disabled / 1-enabled]
	
	--------------------
	-*- Requirements -*-
	--------------------
	
	* Mods: Counter-Strike 1.6 or Condition-Zero
	* AMXX: Version 1.8.0 or later
	* Module: fakemeta, hamsandwich
	
	-----------------
	-*- Changelog -*-
	-----------------
	
	* v1.0.0:
	   - Initial release Privat (13th Aug 2010)
	   - Initial release Alliedmodders (4th Feb 2011)
	
	* v1.1.0:
	   - Added: cvars for nemesis and zombies,
	      human and survivor don't need to
		  disable the engine knockback
	
	* v1.1.1:
	   - Ported to Zombie Plague Nightmare
	   - Added: cvar for assassins
	   - Fixed some potential bugs
	   - Optimized a small code
	
=================================================================================*/

/*
To do:

T = needs testing
X = done
- = cancelled

[T] CZ bots support
v1.1.3
[ ] 
*/

// Modules
#include <amxmodx>
#include <fakemeta>
#include <hamsandwich>

#if AMXX_VERSION_NUM < 180
	#assert AMX Mod X v1.8.0 or later library required!
#endif

// Plug-in APIs
#include <cs_ham_bots_api>
#include <zombieplaguenightmare>

// Stocks
#include <xs>

/*================================================================================
 [Constants, Offsets, Macros]
=================================================================================*/

// Plugin Version
new const PLUGIN_VERSION[] = "1.1.3"

/*================================================================================
 [Global Variables]
=================================================================================*/

// Player vars
new Float:g_Knockback[33][3] // velocity from your knockback position
new g_bEnable[33] // disabled engine knockback is enable

// Cvar pointers
new cvar_Nemesis, cvar_Zombies, cvar_Assassins

/*================================================================================
 [Plugin Initialization]
=================================================================================*/

public plugin_init()
{
	register_plugin("[ZPNM] No Engine Knockback", PLUGIN_VERSION, "schmurgel1983, D i 5 7 i n c T")
	
	RegisterHam(Ham_TakeDamage, "player", "fw_TakeDamage")
	RegisterHam(Ham_TakeDamage, "player", "fw_TakeDamage_Post", 1)
	RegisterHam(Ham_Killed, "player", "fw_PlayerKilled_Post", 1)
	RegisterHamBots(Ham_TakeDamage, "fw_TakeDamage")
	RegisterHamBots(Ham_TakeDamage, "fw_TakeDamage_Post", 1)
	RegisterHamBots(Ham_Killed, "fw_PlayerKilled_Post", 1)
	
	cvar_Nemesis = register_cvar("zpnm_nek_nemesis", "0")
	cvar_Zombies = register_cvar("zpnm_nek_zombies", "1")
	cvar_Assassins = register_cvar("zpnm_nek_assassins", "1")
	
	register_cvar("NoEngineKnockback_version", PLUGIN_VERSION, FCVAR_SERVER|FCVAR_SPONLY)
	set_cvar_string("NoEngineKnockback_version", PLUGIN_VERSION)
}


/*================================================================================
 [Main Forwards]
=================================================================================*/

public client_disconnect(iID)
	g_bEnable[iID] = false

// Ham Take Damage Forward
public fw_TakeDamage(victim)
{
	if (!g_bEnable[victim]) return;
	
	// Engine Knockback disabled
	pev(victim, pev_velocity, g_Knockback[victim])
}

// Ham Take Damage Post Forward
public fw_TakeDamage_Post(victim)
{
	if (!g_bEnable[victim]) return;
	
	// Engine Knockback disabled
	static Float:push[3]
	pev(victim, pev_velocity, push)
	xs_vec_sub(push, g_Knockback[victim], push)
	xs_vec_mul_scalar(push, 0.0, push)
	xs_vec_add(push, g_Knockback[victim], push)
	set_pev(victim, pev_velocity, push)
}

public zpnm_user_spawn_post(iID)
	g_bEnable[iID] = false

public zp_user_humanized_post(iID)
	g_bEnable[iID] = false

public zp_user_infected_post(iID, infector, nemesis, assassin)
{
	if (!assassin && (!nemesis && get_pcvar_num(cvar_Zombies) || nemesis
	&& get_pcvar_num(cvar_Nemesis)) || assassin && get_pcvar_num(cvar_Assassins))
		g_bEnable[iID] = true
	else
		g_bEnable[iID] = false
}

public fw_PlayerKilled_Post(const iID)
	g_bEnable[iID] = false
